// vengeance_misc.h
#ifndef __vengeance_misc_h__
#define __vengeance_misc_h__

#define MAX_TOKEN_LENGTH 1024

float incremental_round( float value, float increment );

bool load_script( XGFileSpecifier *fs );
void free_script( void );
bool parse_token( char *token, bool cross_lines = true, long *line = NULL );
bool parse_float( float *f );
bool parse_int( int *i );
bool seek_to_token( const char *token_to_find, bool case_sensitive = true );

#endif // __vengeance_misc_h__